/*
 * Sound Monitor applet for the GNOME panel
 * (C) 2003 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#include "sound-monitor.h"
#include "update.h"
#include "esdcalls.h"
#include "manager.h"
#include "skin.h"
#include "ui2_editor.h"
#include "ui2_main.h"


static void about_cb(PanelApplet *widget, gpointer data)
{
	static GtkWidget *about = NULL;
	GtkWidget *slikwid;
	GtkWidget *href;

	const gchar *authors[] = {
		"John Ellis <johne@bellatlantic.net>",
		NULL
		};
	const gchar *documenters[] = {
		NULL
		};
	const gchar *translators = _("translators");

	if (about != NULL)
		{
		gdk_window_show(about->window);
		gdk_window_raise(about->window);
		return;
		}

        about = gnome_about_new ( _("Sound Monitor Applet"), VERSION,
			"(C) 2003 John Ellis",
			_("Sound monitor interface to Esound\n\n"
			"Released under the GNU general public license."),
			authors, documenters,
			strcmp (translators, "translators") != 0 ? translators : NULL,
			NULL);

	href = gnome_href_new("http://gqapplets.sourceforge.net", _("Sound Monitor home page"));
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(about)->vbox), href, FALSE, FALSE, 0);
	gtk_widget_show(href);

	slikwid = ui_slik_credit();
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(about)->vbox), slikwid, FALSE, FALSE, 0);
	gtk_widget_show(slikwid);

	gtk_window_set_wmclass(GTK_WINDOW(about), "sound_monitor", "Sound Monitor");

	gtk_signal_connect(GTK_OBJECT(about), "destroy",
			   GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about);
	gtk_widget_show(about);
}

#if 0
static void esd_control_cb (PanelApplet *widget, gpointer data)
{
	AppData *ad = data;

	if (ad->esd_status == Status_Error)
		{
		if (esd_control(Control_Start, ad->esd_host));
		sound_free(ad->sound);
		ad->sound = sound_init(ad->esd_host, FALSE);
		sound_set_latency(ad->sound, ad->latency);
		}
	else if (ad->esd_status == Status_Standby)
		{
		esd_control(Control_Resume, ad->esd_host);
		}
	else if (ad->esd_status == Status_Ready)
		{
		esd_control(Control_Standby, ad->esd_host);
		}
	return;
}
#endif

static void manager_cb (PanelApplet *widget, gpointer data)
{
	AppData *ad = data;
	manager_window_show(ad);
	return;
}

void sync_esd_menu_item(AppData *ad)
{
	gchar *menu_text = "";
	gchar *stock_type;
	switch (ad->esd_status)
		{
		case Status_Ready:
		case Status_AutoStandby:
			menu_text = _("Place Esound in standby");
			stock_type = GNOME_STOCK_MENU_STOP;
			break;
		case Status_Standby:
			menu_text = _("Resume Esound");
			stock_type = GNOME_STOCK_MENU_VOLUME;
			break;
		default:
			menu_text = _("Start Esound");
			stock_type = GNOME_STOCK_MENU_VOLUME;
			break;
		}

#if 0
	applet_widget_unregister_callback (APPLET_WIDGET(ad->applet),
						"control");
	applet_widget_register_stock_callback(APPLET_WIDGET(ad->applet),
						"control",
						stock_type,
						menu_text,
						esd_control_cb,
						ad);
#endif
	ad->esd_status_menu = ad->esd_status;
}

void reload_skin(AppData *ad)
{
	if (ad->theme_file && strlen(ad->theme_file) == 0)
		{
		skin_set(NULL, ad);
		}
	else if (!skin_set(ad->theme_file, ad))
		{
		printf("Failed to load skin %s, loading default\n", ad->theme_file);
		skin_set(NULL, ad);
		}
}

/* clean up function to free all the applet's memory and stop io,timers */
static void destroy_applet(GtkWidget *widget, gpointer data)
{
	AppData *ad = data;

	manager_window_close(ad);

	gtk_timeout_remove(ad->update_timeout_id);

	sound_free(ad->sound);

	property_save();

	g_free(ad->esd_host);
	g_free(ad->theme_file);
	g_free(ad);
}

static void applet_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer data)
{
	AppData *ad = data;

	ui_geometry_get(ad->ui, NULL, NULL, NULL, NULL);
}

static void applet_change_orient(GtkWidget *w, PanelAppletOrient o, gpointer data)
{
	AppData *ad = data;
	ad->orient = o;

	if (ad->ui->skin) reload_skin(ad);
}

static gint size_hint_from_size(gint size)
{
	if (size < 32)
		return SIZEHINT_TINY;
	else if (size < 48)
		return SIZEHINT_SMALL;
	else if (size < 64)
		return SIZEHINT_STANDARD;
	else if (size < 80)
		return SIZEHINT_LARGE;
	else
		return SIZEHINT_HUGE;
}

static void applet_change_size(GtkWidget *w, int size, gpointer data)
{
	AppData *ad = data;

	ad->sizehint = size_hint_from_size(size);

	if (ad->ui->skin) reload_skin(ad);
}

static void applet_do_draw(GtkWidget *w, PanelAppletBackgroundType  type, GdkColor *color, GdkPixmap *pixmap, gpointer data)
{
	AppData *ad = data;

	applet_skin_backing_sync(ad);
}

static void help_cb(PanelApplet *applet, gpointer data)
{
#if 0
	GnomeHelpMenuEntry help_entry = { "sound-monitor_applet2", 
					  "index.html" };
	gnome_help_display(NULL, &help_entry);
#endif
}

static void editor_cb(PanelApplet *applet, gpointer data)
{
	AppData *ad = data;
	EditData *ed;

	ed = ui_edit_new(ad->ui, "Sound-Monitor", "skin_editor", _("Skin editor - Sound Monitor"));
	if (ed)
		{
		GList *list = NULL;
		gchar *help_path;

		list = g_list_append(list, "skindata_h");
		list = g_list_append(list, "skindata_h_tiny");
		list = g_list_append(list, "skindata_h_small");
		list = g_list_append(list, "skindata_h_large");
		list = g_list_append(list, "skindata_h_huge");
		list = g_list_append(list, "skindata_v");
		list = g_list_append(list, "skindata_v_tiny");
		list = g_list_append(list, "skindata_v_small");
		list = g_list_append(list, "skindata_v_large");
		list = g_list_append(list, "skindata_v_huge");

		ui_edit_set_skindata_list(ed, list);
		g_list_free(list);

		help_path = g_strconcat(SOUND_MONITOR_THEME_PATH,  "/SKIN-SPECS", NULL);
		ui_edit_set_help_path(ed, help_path);
		g_free(help_path);
		}
}

/* popup menu */
static const BonoboUIVerb applet_menu_verbs[] = {
	BONOBO_UI_UNSAFE_VERB("Manager", manager_cb),
	BONOBO_UI_UNSAFE_VERB("Properties", property_show),
	BONOBO_UI_UNSAFE_VERB("Editor", editor_cb),
	BONOBO_UI_UNSAFE_VERB("Help", help_cb),
	BONOBO_UI_UNSAFE_VERB("About", about_cb),
	BONOBO_UI_VERB_END
};

static gint applet_widget_new(PanelApplet *applet)
{
        AppData *ad;
	static gint init_done = FALSE;

	if (!init_done)
		{
		/* set the SLIK options here */
		slik_remember_position = TRUE;
		slik_allow_shapes = FALSE;
		slik_focus_enable = TRUE;

		ui_edit_add_skin_resource(SOUND_MONITOR_THEME_PATH);

		init_done = TRUE;
		}

	gnome_window_icon_set_default_from_file(GNOME_ICONDIR"/sound-monitor.png");

	panel_applet_add_preferences(applet, "/schemas/apps/sound-monitor-applet/prefs", NULL);

        ad = g_new0(AppData, 1);

	ad->applet = applet;
	ad->propwindow = NULL;
	ad->orient = panel_applet_get_orient(applet);
	ad->sizehint = size_hint_from_size(panel_applet_get_size(applet));

	ad->esd_status = Status_Error;
	ad->esd_status_menu = Status_Error;

	ad->sound = NULL;
	ad->latency = -1;

	ad->esd_host = NULL;

	ad->refresh_fps = 40;
	ad->falloff_speed = 5;
	ad->peak_mode = PeakMode_Active;
	ad->scope_scale = 20;
	ad->draw_scope_as_segments = TRUE;

	ad->theme_file = NULL;

	property_load(ad);

	gtk_signal_connect(GTK_OBJECT(ad->applet), "destroy",
			   GTK_SIGNAL_FUNC(destroy_applet), ad);

	/* use this signal to sync the applet's geomtry when the size changes */
	gtk_signal_connect(GTK_OBJECT(ad->applet), "size_allocate",
			   GTK_SIGNAL_FUNC(applet_size_allocate), ad);

	ad->ui = ui_new_into_container("sound-monitor", "applet", GTK_WIDGET(applet));
	ad->ui->window = GTK_WIDGET(applet);

	display_register_widgets(ad);
	ui_set_skin_callback(ad->ui, applet_skin_default, ad);
	ui_set_back_callback(ad->ui, applet_skin_back_set_cb, ad);

	gtk_signal_connect(GTK_OBJECT(ad->applet),"change_orient",
			   GTK_SIGNAL_FUNC(applet_change_orient), ad);

	gtk_signal_connect(GTK_OBJECT(ad->applet),"change_size",
		GTK_SIGNAL_FUNC(applet_change_size), ad);

	gtk_signal_connect(GTK_OBJECT(ad->applet),"change_background",
		GTK_SIGNAL_FUNC(applet_do_draw), ad);

	panel_applet_setup_menu_from_file(applet, NULL, "GNOME_SoundMonitorApplet.xml",
					  NULL, applet_menu_verbs, ad);
	gtk_widget_show(GTK_WIDGET(ad->applet));

	reload_skin(ad);

	update_display(ad);

	ad->sound = sound_init(ad->esd_host, FALSE);
	sound_set_latency(ad->sound, ad->latency);

	ad->update_timeout_id = gtk_timeout_add(1000 / ad->refresh_fps, (GtkFunction)update_display, ad);
	
	return TRUE;
}

static gboolean applet_factory(PanelApplet *applet, const gchar *iid, gpointer data)
{
	gboolean retval = FALSE;

	if (strcmp(iid, "OAFIID:GNOME_SoundMonitorApplet") == 0)
       		{
                retval = applet_widget_new(applet);
	        }

	return retval;
}

PANEL_APPLET_BONOBO_FACTORY("OAFIID:GNOME_SoundMonitorApplet_Factory", PANEL_TYPE_APPLET,
			    "Sound-Monitor-Applet", "0", applet_factory, NULL)

