/* 
 * Sound Monitor applet for the GNOME panel
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#include "sound-monitor.h"
#include "skin.h"
#include "update.h"
#include "ui_pixbuf_ops.h"
#include "ui2_main.h"
#include "ui2_skin.h"
#include "ui2_parse.h"

#include "ui2_item.h"
#include "ui2_meter.h"
#include "ui2_scope.h"
#include "ui2_spectrum.h"

/* the default pixmaps */
#include "def_theme/back_h.xpm"
#include "def_theme/led_h.xpm"
#include "def_theme/status_h.xpm"
#include "def_theme/back_v.xpm"
#include "def_theme/led_v.xpm"
#include "def_theme/status_v.xpm"

#include "def_theme/focus.xpm"

SkinData *applet_skin_default(UIData *ui, const gchar *key, gpointer data)
{
	SkinData *skin;

	skin = skin_new();

	if (key && (strcmp(key, "skindata_h_tiny") == 0 ||
		    strcmp(key, "skindata_h_small") == 0 ||
		    strcmp(key, "skindata_v") == 0 ||
		    strcmp(key, "skindata_v_large") == 0 ||
		    strcmp(key, "skindata_v_huge") == 0 ))
		{
		skin->real_overlay = gdk_pixbuf_new_from_xpm_data((const char **)back_h_xpm);

		meter_register_to_skin("left", (gchar **)led_h_xpm, 5, 1,
				       14, TRUE, FALSE, skin, NULL);
		meter_register_to_skin("right", (gchar **)led_h_xpm, 5, 10,
				       14, TRUE, FALSE, skin, NULL);

		item_register_to_skin("status", (gchar **)status_v_xpm, 1, 1,
				      4, skin, NULL);
		}
	else
		{
		skin->real_overlay = gdk_pixbuf_new_from_xpm_data((const char **)back_v_xpm);

		meter_register_to_skin("left", (gchar **)led_v_xpm, 1, 1,
				       14, FALSE, FALSE, skin, NULL);
		meter_register_to_skin("right", (gchar **)led_v_xpm, 10, 1,
				       14, FALSE, FALSE, skin, NULL);

		item_register_to_skin("status", (gchar **)status_h_xpm, 1, 45,
				      4, skin, NULL);
		}

	skin->width = gdk_pixbuf_get_width(skin->real_overlay);
	skin->height = gdk_pixbuf_get_height(skin->real_overlay);

	skin->width_def = skin->width_min = skin->width_max = skin->width;
	skin->height_def = skin->height_min = skin->height_max = skin->height;

	return skin;
}

#if 0
static gint applet_skin_backing_sync_idle(gpointer data)
{
	AppData *ad = data;

	applet_skin_backing_sync(ad);

	return FALSE;
}
#endif

static gint debug_window_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
{
	GtkWidget **window = data;

	gtk_widget_destroy(*window);
	*window = NULL;

	return TRUE;
}

static GdkPixbuf *applet_skin_get_back(AppData *ad)
{
	PanelAppletBackgroundType type;
	GdkColor color;
	GdkPixmap *pixmap;

	type = panel_applet_get_background(ad->applet, &color, &pixmap);

	if (type == PANEL_COLOR_BACKGROUND || type == PANEL_NO_BACKGROUND)
		{
		GdkPixbuf *pb;
		gint r, g, b;
		gint pw, ph;

		if (type == PANEL_COLOR_BACKGROUND)
			{
			r = color.red >> 8;
			g = color.green >> 8;
			b = color.blue >> 8;
			}
		else
			{
			GtkStyle *style;
			GdkColor *bg;

			style = gtk_widget_get_style(GTK_WIDGET(ad->applet));
			bg = &style->bg[GTK_STATE_NORMAL];
			r = bg->red >> 8;
			g = bg->green >> 8;
			b = bg->blue >> 8;
			}

		pw = gdk_pixbuf_get_width(ad->ui->skin->pixbuf);
		ph = gdk_pixbuf_get_height(ad->ui->skin->pixbuf);

		pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, pw, ph);
		pixbuf_draw_rect_fill(pb, 0, 0, pw, ph, r, g, b, 255);

		return pb;
		}
	else if (type == PANEL_PIXMAP_BACKGROUND)
		{
		GdkPixbuf *pb;
		gint pw, ph;
		gint x, y;
		GdkColormap *cmap;

                pw = gdk_pixbuf_get_width(ad->ui->skin->pixbuf);
		ph = gdk_pixbuf_get_height(ad->ui->skin->pixbuf);

		pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, pw, ph);
		
		cmap = gdk_window_get_colormap(GTK_WIDGET(ad->applet)->window);
		x = 0;
		y = 0;
		gdk_pixbuf_get_from_drawable(pb, pixmap, cmap, x, y, 0, 0, pw, ph);

		/* test debugging window to see what panel is passing the applet */
		if (pixmap && debug_mode)
			{
			static GtkWidget *window = NULL;
			static GtkWidget *image = NULL;
			static GtkWidget *counter = NULL;
			static gint count = 0;
			gchar *text;
			
			if (!window)
				{
				GtkWidget *vbox;
				window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
				gtk_signal_connect(GTK_OBJECT(window), "delete_event",
						   GTK_SIGNAL_FUNC(debug_window_delete_cb), &window);

				vbox = gtk_vbox_new(FALSE, 0);
				gtk_container_add(GTK_CONTAINER(window), vbox);
				gtk_widget_show(vbox);

				image = gtk_image_new();
				gtk_box_pack_start(GTK_BOX(vbox), image, FALSE, FALSE, 0);
				gtk_widget_show(image);

				counter = gtk_label_new("");
				gtk_box_pack_start(GTK_BOX(vbox), counter, FALSE, FALSE, 0);
				gtk_widget_show(counter);

				gtk_widget_show(window);
				}

			gtk_image_set_from_pixmap(GTK_IMAGE(image), pixmap, NULL);

			count++;
			text = g_strdup_printf("count: %d", count);
			gtk_label_set_text(GTK_LABEL(counter), text);
			g_free(text);
			}

		gdk_pixmap_unref(pixmap);

		return pb;		
		}
	else
		{
		g_print("Unknown panel background type\n");
		}

	if (pixmap) gdk_pixmap_unref(pixmap);

	return NULL;
}

void applet_skin_backing_sync(AppData *ad)
{
	GdkPixbuf *pb;

	if (!ad->ui || !ad->ui->skin || !ad->ui->skin->pixbuf) return;

	if (ad->ui->skin->transparent)
		{
		skin_sync_back(ad->ui->skin, ad->ui);
		return;
		}

	if (!gdk_pixbuf_get_has_alpha(ad->ui->skin->real_overlay)) return;

	pb = applet_skin_get_back(ad);

	if (pb)
		{
		ui_set_underlay(ad->ui, pb);
		gdk_pixbuf_unref(pb);
		}
}

gint applet_skin_back_set_cb(UIData *ui, GdkPixbuf *pb, gpointer data)
{
	AppData *ad = data;
	GdkPixbuf *new_pb;

	new_pb = applet_skin_get_back(ad);

	if (new_pb)
		{
		gint w, h;

		w = gdk_pixbuf_get_width(pb);
		h = gdk_pixbuf_get_height(pb);

		pixbuf_copy_area(new_pb, 0, 0, pb, 0, 0, w, h, FALSE);
		gdk_pixbuf_unref(new_pb);
		}
	else
		{
		printf("warning: no backing pb\n");
		return FALSE;
		}

	return TRUE;
}

static gchar *skin_mode_key(gint vertical, gint size)
{
	const gchar *stext;

	switch (size)
		{
		case SIZEHINT_TINY:
			stext = "_tiny";
			break;
		case SIZEHINT_SMALL:
			stext = "_small";
			break;
		case SIZEHINT_LARGE:
			stext = "_large";
			break;
		case SIZEHINT_HUGE:
			stext = "_huge";
			break;
		case SIZEHINT_STANDARD:
		default:
			stext = "";
			break;
		}

	return g_strdup_printf("skindata%s%s", (vertical) ? "_v" : "_h", stext);
}

static SkinData *skin_load(const gchar *skin_path, gint vertical, gint size, gint edit, gchar **key)
{
	SkinData *skin;
	gchar *buf;

	*key = skin_mode_key(vertical, size);
	buf = g_strconcat(skin_path, "/", *key, NULL);
	skin = skin_parse(skin_path, buf, edit);
	g_free(buf);

	if (!skin)
		{
		g_free(*key);
		*key = NULL;
		}

	return skin;
}

static SkinData *skin_load_best_match(const gchar *path, gint vertical, gint size, gint edit, gchar **key)
{
	SkinData *s = NULL;

	s = skin_load(path, vertical, size, edit, key);
	if (s) return s;

	/* tiny falls back to small */
	if (size == SIZEHINT_TINY)
		{
		s = skin_load(path, vertical, SIZEHINT_SMALL, edit, key);
		if (s) return s;
		}

	/* small falls back to tiny */
	if (size == SIZEHINT_SMALL)
		{
		s = skin_load(path, vertical, SIZEHINT_TINY, edit, key);
		if (s) return s;
		}

	/* huge falls back to large */
	if (size == SIZEHINT_HUGE)
		{
		s = skin_load(path, vertical, SIZEHINT_LARGE, edit, key);
		if (s) return s;
		}

	if (size == SIZEHINT_STANDARD) return NULL;

	/* failed so far, try standard */
	return skin_load(path, vertical, SIZEHINT_STANDARD, edit, key);
}

gint skin_set(const gchar *path, AppData *ad)
{
	SkinData *new_s = NULL;
	gchar *key = NULL;

	if (!path)
		{
		key = skin_mode_key((ad->orient == PANEL_APPLET_ORIENT_LEFT || ad->orient == PANEL_APPLET_ORIENT_RIGHT), ad->sizehint);
		new_s = ui_skin_load_default(ad->ui, key);
		}
	else
		{
		new_s = skin_load_best_match(path,
					     (ad->orient == PANEL_APPLET_ORIENT_LEFT || ad->orient == PANEL_APPLET_ORIENT_RIGHT),
					     ad->sizehint, FALSE, &key);
		if (!new_s)
			{
			new_s = skin_load(path,
					  !(ad->orient == PANEL_APPLET_ORIENT_LEFT || ad->orient == PANEL_APPLET_ORIENT_RIGHT),
					  SIZEHINT_STANDARD, FALSE, &key);
			}
		}

	if (!new_s)
		{
		g_free(key);
		return FALSE;	
		}

	/* override the focus to something sane, if not defined by skin */
	if (!new_s->focus_filename && !new_s->focus_overlay &&
	    !new_s->focus_box_filled &&
	    new_s->focus_box_r == 255 && new_s->focus_box_g == 0 && new_s->focus_box_b == 0 &&
	    new_s->focus_box_alpha == 128)
		{
		new_s->focus_filename = NULL;
		new_s->focus_overlay = gdk_pixbuf_new_from_xpm_data((const char **)focus_xpm);
		new_s->focus_stretch = FALSE;
		new_s->focus_has_border = TRUE;
		new_s->focus_border_left = 1;
		new_s->focus_border_right = 1;
		new_s->focus_border_top = 1;
		new_s->focus_border_bottom = 1;
		new_s->focus_anchor_right = FALSE;
		new_s->focus_anchor_bottom = FALSE;
		new_s->focus_box_alpha = 170;
		}

	ui_skin_set(ad->ui, new_s, path, key);
	applet_skin_backing_sync(ad);

	ad->has_meter = (ui_widget_exists(ad->ui, "main", meter_type_id()) ||
			 ui_widget_exists(ad->ui, "left", meter_type_id()) ||
			 ui_widget_exists(ad->ui, "right", meter_type_id()) ||
			 ui_widget_exists(ad->ui, "main", item_type_id()) ||
			 ui_widget_exists(ad->ui, "left", item_type_id()) ||
			 ui_widget_exists(ad->ui, "right", item_type_id()) );

	ad->has_scope = ui_widget_exists(ad->ui, "main", scope_type_id());

	ad->has_spectrum = (ui_widget_exists(ad->ui, "main", spectrum_type_id()) ||
			    ui_widget_exists(ad->ui, "left", spectrum_type_id()) ||
			    ui_widget_exists(ad->ui, "right", spectrum_type_id()) );


	g_free(key);

	return TRUE;
}
