/*
 * (SLIK) SimpLIstic sKin functions
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#ifndef UI2_METER_H
#define UI2_METER_H


typedef enum {
	METER_PEAK_OFF = 0,
	METER_PEAK_ACTIVE,
	METER_PEAK_SMOOTH
} MeterPeakType;


typedef struct _MeterData MeterData;
struct _MeterData
{
	GdkPixbuf *overlay;
	GdkPixbuf *pixbuf;
	gint width;
	gint height;
	gint x;
	gint y;

	gint segments;
	gint segment_size;
	gint horizontal;

	gfloat value;

	MeterPeakType peak_mode;
	gfloat peak_value;
	gfloat peak_fall;

	gint slide;
};

#define meter_set_peak_mode(x, y) (x->peak_mode = y)
#define meter_set_peak_fall(x, y) (x->peak_fall = CLAMP(y, 0.1, 10.0))


MeterData *meter_new(GdkPixbuf *pb, gint x, gint y,
		     gint segment, gint horizontal, gint slide);
MeterData *meter_new_from_data(gchar **data, gint x, gint y,
		     gint segments, gint horizontal, gint slide);
MeterData *meter_new_from_file(const gchar *file, gint x, gint y,
		     gint segments, gint horizontal, gint slide);
void meter_free(MeterData *meter);

WidgetData *meter_register(SkinData *skin, MeterData *meter, const gchar *key, const gchar *text_id);
WidgetData *meter_register_to_skin(const gchar *key, gchar **data, gint x, gint y,
				   gint sections, gint horizontal, gint slide,
				   SkinData *skin, const gchar *text_id);

RegisterData *meter_register_key(const gchar *key, UIData *ui,
				 gfloat (*status_get_func)(MeterData *meter, const gchar *key, gpointer data), gpointer status_get_data,
				 void (*mode_get_func)(MeterData *meter, const gchar *key, gpointer data), gpointer mode_get_data);

gfloat meter_value_get(const gchar *key, UIData *ui);
gint meter_value_set(const gchar *key, UIData *ui, gfloat value, gfloat fall_percent);

gint meter_reset(const gchar *key, UIData *ui);

gint meter_set_options(const gchar *key, UIData *ui, MeterPeakType mode, gfloat fall_speed);



WidgetType meter_type_id(void);
void meter_type_init(void);


#endif

