/*
 * (SLIK) SimpLIstic sKin functions
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#include "ui2_includes.h"
#include "ui2_typedefs.h"

#include "ui2_scope.h"
#include "ui2_scope_edit.h"

#include "ui2_editor.h"
#include "ui2_widget.h"
#include "ui_fileops.h"
#include "ui_tabcomp.h"
#include "ui_utildlg.h"


typedef struct _ScopeDetail ScopeDetail;
struct _ScopeDetail
{
	ScopeData *scope;
	GtkWidget *horizontal_toggle;
	GtkWidget *dynamic_toggle;
	GtkWidget *stretch_toggle;
	GtkWidget *width_spin;
	GtkWidget *height_spin;

	UIData *ui;
	WidgetData *wd;
};

typedef struct _ScopeListData ScopeListData;
struct _ScopeListData
{
	gchar *key;
	gchar *text_id;
	gchar *data;

	gchar *image;
	gint horizontal;
	gint dynamic;
	gint stretch;
};

typedef struct _ScopePage ScopePage;
struct _ScopePage
{
	GtkWidget *key_entry;
	GtkWidget *text_id_entry;
	GtkWidget *data_entry;

	GtkWidget *image_entry;
	GtkWidget *horizontal_toggle;
	GtkWidget *dynamic_toggle;
	GtkWidget *stretch_toggle;

	GtkWidget *clist;
	GtkWidget *pixmap;

	EditData *ed;
};


static GdkPixbuf *scope_get_pixbuf(gpointer data)
{
	ScopeData *scope = data;
	return scope->overlay;
}

static void scope_edit_write(FILE *f, WidgetData *wd, SkinData *skin, const gchar *dir)
{
	ScopeData *scope = wd->widget;
	gchar *image;

	image = ui_edit_copy_unique_file(ui_widget_get_data(wd, "image"),
					 dir, scope->overlay, "scope_", wd->key);

	if (image) ui_edit_widget_set_path_key(wd, "image", dir, image);

	ui_edit_write_section(f, "scope", wd->key);

	ui_edit_write_key_char(f, "image", image);
	ui_edit_write_key_int(f, "x", scope->x);
	ui_edit_write_key_int(f, "y", scope->y);
	ui_edit_write_key_int(f, "width", scope->width);
	ui_edit_write_key_int(f, "height", scope->height);
	ui_edit_write_key_bool(f, "horizontal", scope->horizontal);
	ui_edit_write_key_bool(f, "dynamic", scope->dynamic);
	ui_edit_write_key_bool(f, "stretch", scope->stretch);

	g_free(image);
}

static ScopeListData *scope_edit_list_find(GList *list, const gchar *image)
{
	GList *work;
	work = list;
	while (work)
		{
		ScopeListData *sd = work->data;
		if (strcmp(image, sd->image) == 0) return sd;
		work = work->next;
		}
	return NULL;
}

static gpointer scope_edit_read(UIData *ui, WidgetData *wd, GList *list)
{
	ScopeData *scope = wd->widget;
	ScopeListData *sd;
	const gchar *image;

	image = ui_widget_get_data(wd, "image");

	if (!image || scope_edit_list_find(list, image)) return NULL;

	sd = g_new0(ScopeListData, 1);
	sd->image = g_strdup(image);
	sd->horizontal = scope->horizontal;
	sd->dynamic = scope->dynamic;
	sd->stretch = scope->stretch;

	sd->key = g_strdup(wd->key);
	sd->data = g_strdup(ui_widget_get_data(wd, "data"));
	sd->text_id = g_strdup(wd->text_id);

	return sd;
}

static void scope_edit_props_horizontal_cb(GtkWidget *widget, gpointer data)
{
	ScopeDetail *sd = data;

	scope_sync_to_size(sd->scope, sd->scope->width, sd->scope->height,
			   ui_edit_toggle_get(sd->horizontal_toggle));
	ui_edit_widget_resync(sd->ui, sd->wd, TRUE, 0, 0);
}

static void scope_edit_props_dynamic_cb(GtkWidget *widget, gpointer data)
{
	ScopeDetail *sd = data;

	sd->scope->dynamic = ui_edit_toggle_get(sd->dynamic_toggle);
}

static void scope_edit_props_stretch_cb(GtkWidget *widget, gpointer data)
{
	ScopeDetail *sd = data;

	sd->scope->stretch = ui_edit_toggle_get(sd->stretch_toggle);
	ui_edit_widget_resync(sd->ui, sd->wd, TRUE, 0, 0);
}

static void scope_edit_props_size_do(ScopeDetail *sd, gint w, gint h)
{
	gint dw, dh;

	if (sd->scope->x + w > sd->ui->skin->width)
		{
		ui_widget_set_coord(sd->ui, sd->wd, MAX(0, sd->ui->skin->width - w), sd->scope->y, TRUE);
		}
	if (sd->scope->y + h > sd->ui->skin->height)
		{
		ui_widget_set_coord(sd->ui, sd->wd, sd->scope->x, MAX(0, sd->ui->skin->height - h), TRUE);
		}
	if (sd->scope->x + w > sd->ui->skin->width)
		{
		w = sd->ui->skin->width;
		ui_edit_spin_set_blocking(sd->width_spin, w, sd);
		}
	if (sd->scope->y + h > sd->ui->skin->height)
		{
		h = sd->ui->skin->height;
		ui_edit_spin_set_blocking(sd->height_spin, h, sd);
		}

	dw = sd->scope->width - w;
	dh = sd->scope->height - h;

	ui_edit_widget_draw_highlight(sd->ui, sd->wd, FALSE);
	scope_sync_to_size(sd->scope, w, h, sd->scope->horizontal);
	ui_edit_widget_resync(sd->ui, sd->wd, FALSE, dw, dh);
	ui_edit_widget_draw_highlight(sd->ui, sd->wd, TRUE);
}

static void scope_edit_props_size_cb(GtkAdjustment *adj, gpointer data)
{
	ScopeDetail *sd = data;
	gint w, h;

	w = ui_edit_spin_get(sd->width_spin);
	h = ui_edit_spin_get(sd->height_spin);

	scope_edit_props_size_do(sd, w, h);
}

static void scope_edit_props_reset_cb(GtkWidget *widget, gpointer data)
{
	ScopeDetail *sd = data;
	gint w, h;

	w = gdk_pixbuf_get_width(sd->scope->overlay);
	h = gdk_pixbuf_get_height(sd->scope->overlay) / 2;

	ui_edit_spin_set_blocking(sd->width_spin, w, sd);
	ui_edit_spin_set_blocking(sd->height_spin, h, sd);

	scope_edit_props_size_do(sd, w, h);
}

static gpointer scope_edit_props(UIData *ui, WidgetData *wd, GtkWidget *vbox, gpointer detail)
{
	ScopeData *scope = wd->widget;
	ScopeDetail *sd = detail;

	if (!sd)
		{
		GtkObject *adj;
		GtkWidget *hbox;
		GtkWidget *button;

		sd = g_new0(ScopeDetail, 1);

		hbox = ui_edit_frame_new(vbox, TRUE, _("Size"));
		sd->width_spin = ui_edit_spin_new(hbox, _("Width:"), 4, 2000, &adj);
		gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
				   GTK_SIGNAL_FUNC(scope_edit_props_size_cb), sd);
		sd->height_spin = ui_edit_spin_new(hbox, _("Height:"), 4, 2000, &adj);
		gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
				   GTK_SIGNAL_FUNC(scope_edit_props_size_cb), sd);
		button = gtk_button_new_with_label(_("Reset to image"));
		gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
		gtk_widget_show(button);
		gtk_signal_connect(GTK_OBJECT(button), "clicked",
				   GTK_SIGNAL_FUNC(scope_edit_props_reset_cb), sd);

		sd->horizontal_toggle = ui_edit_toggle_new(vbox, _("Horizontal"));
		gtk_signal_connect(GTK_OBJECT(sd->horizontal_toggle), "clicked",
				   GTK_SIGNAL_FUNC(scope_edit_props_horizontal_cb), sd);
		sd->dynamic_toggle = ui_edit_toggle_new(vbox, _("Dynamic size"));
		gtk_signal_connect(GTK_OBJECT(sd->dynamic_toggle), "clicked",
				   GTK_SIGNAL_FUNC(scope_edit_props_dynamic_cb), sd);
		sd->stretch_toggle = ui_edit_toggle_new(vbox, _("Stretch"));
		gtk_signal_connect(GTK_OBJECT(sd->stretch_toggle), "clicked",
				   GTK_SIGNAL_FUNC(scope_edit_props_stretch_cb), sd);
		}

	sd->scope = scope;
	sd->wd = wd;
	sd->ui = ui;


	ui_edit_spin_set_blocking(sd->width_spin, scope->width, sd);
	ui_edit_spin_set_blocking(sd->height_spin, scope->height, sd);

	ui_edit_toggle_set(sd->horizontal_toggle, scope->horizontal);
	ui_edit_toggle_set(sd->dynamic_toggle, scope->dynamic);
	ui_edit_toggle_set(sd->stretch_toggle, scope->stretch);

	return sd;
}

static void scope_edit_free(gpointer data)
{
	ScopeListData *sd = data;
	g_free(sd->key);
	g_free(sd->text_id);
	g_free(sd->data);
	g_free(sd->image);
	g_free(sd);
}

static void scope_edit_page_add_cb(GtkWidget *widget, gpointer data)
{
	ScopePage *pd = data;
	ScopeData *scope;
	const gchar *key;
	const gchar *text_id;
	const gchar *image;

	key = ui_edit_entry_get(pd->key_entry);
	text_id = ui_edit_entry_get(pd->text_id_entry);
	image = ui_edit_entry_get(pd->image_entry);

	if (!key || !image || !isfile(image))
		{
		warning_dialog(_("Scope error"), _("Scope must contain a key and valid image."));
		return;
		}

	scope = scope_new_from_file(image, 0, 0,
				    ui_edit_toggle_get(pd->horizontal_toggle),
				    -1, -1,
				    ui_edit_toggle_get(pd->dynamic_toggle),
				    ui_edit_toggle_get(pd->stretch_toggle));
	if (scope)
		{
		WidgetData *wd;
		wd = scope_register(pd->ed->ui->skin, scope, key, text_id);
		ui_edit_widget_add_finish(pd->ed, wd, image, ui_edit_entry_get(pd->data_entry));
		}

	tab_completion_append_to_history(pd->image_entry, image);
}

static void scope_edit_page_sync(ScopePage *pd, ScopeListData *sd)
{
	if (!sd) return;

	ui_edit_entry_set(pd->key_entry, sd->key);
	ui_edit_entry_set(pd->text_id_entry, sd->text_id);
	ui_edit_entry_set(pd->data_entry, sd->data);

	ui_edit_entry_set(pd->image_entry, sd->image);

	ui_edit_toggle_set(pd->horizontal_toggle, sd->horizontal);
	ui_edit_toggle_set(pd->dynamic_toggle, sd->dynamic);
	ui_edit_toggle_set(pd->stretch_toggle, sd->stretch);
}

static void scope_edit_page_clist_cb(GtkWidget *clist, gint row, gint col, GdkEvent *event, gpointer data)
{
	ScopePage *pd = data;
	ScopeListData *sd;

	sd = gtk_clist_get_row_data(GTK_CLIST(clist), row);
	scope_edit_page_sync(pd, sd);
}

static void scope_edit_page_destroy_cb(GtkWidget *widget, gpointer data)
{
	ScopePage *pd = data;

	g_free(pd);
}

static GtkWidget *scope_edit_page_new(EditData *ed)
{
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *scope;
	ScopePage *pd;
	gchar *titles[] = { _("Image"), _("Key"), NULL };

	pd = g_new0(ScopePage, 1);
	pd->ed = ed;

	hbox = gtk_hbox_new(FALSE, 5);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
	gtk_object_set_data(GTK_OBJECT(hbox), "page", pd);
	gtk_signal_connect(GTK_OBJECT(hbox), "destroy",
			   GTK_SIGNAL_FUNC(scope_edit_page_destroy_cb), pd);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
	gtk_widget_show(vbox);

	pd->key_entry = ui_edit_key_entry_new(vbox, ed->ui, scope_type_id());
	pd->data_entry = ui_edit_entry_new(vbox, _("Data:"));
	pd->text_id_entry = ui_edit_entry_new(vbox, _("Text id:"));

	pd->horizontal_toggle = ui_edit_toggle_new(vbox, _("Horizontal"));
	pd->dynamic_toggle = ui_edit_toggle_new(vbox, _("Dynamic size"));
	pd->stretch_toggle = ui_edit_toggle_new(vbox, _("Stretch"));

	pd->image_entry = ui_edit_path_entry_new(vbox, _("Image:"), "SLIK_scope_image");
	pd->pixmap = ui_edit_pixmap_new(vbox);
	ui_edit_path_entry_connect_pixmap(pd->image_entry, pd->pixmap);

	scope = gtk_button_new_with_label(_("Add"));
	gtk_box_pack_start(GTK_BOX(vbox), scope, FALSE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(scope), "clicked",
			   GTK_SIGNAL_FUNC(scope_edit_page_add_cb), pd);
	gtk_widget_show(scope);

	pd->clist = ui_edit_clist_new(hbox, titles, 2);
	gtk_signal_connect(GTK_OBJECT(pd->clist), "select_row",
			   GTK_SIGNAL_FUNC(scope_edit_page_clist_cb), pd);

	gtk_widget_show(hbox);

	return hbox;
}

static void scope_edit_page_add(GtkWidget *widget, gpointer data)
{
	ScopeListData *id = data;
	ScopePage *pd;
	gchar *buf[3];

	pd = gtk_object_get_data(GTK_OBJECT(widget), "page");

	buf[0] = "";
	buf[1] = id->key;
	buf[2] = NULL;

	ui_edit_clist_append(pd->clist, buf, id->image, id);
}

void scope_type_init_edit(WidgetObjectData *od)
{
	od->func_get_pixbuf = scope_get_pixbuf;

	od->func_edit_write = scope_edit_write;

	od->func_edit_read = scope_edit_read;
	od->func_edit_free = scope_edit_free;

	od->func_edit_props = scope_edit_props;

	od->func_edit_page_new = scope_edit_page_new;
	od->func_edit_page_add = scope_edit_page_add;
}


