/*
 * (SLIK) SimpLIstic sKin functions
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#ifndef UI2_SPECTRUM_H
#define UI2_SPECTRUM_H


typedef enum {
	SPECTRUM_PEAK_OFF = 0,
	SPECTRUM_PEAK_ACTIVE,
	SPECTRUM_PEAK_SMOOTH
} SpectrumPeakType;


typedef struct _SpectrumData SpectrumData;
struct _SpectrumData
{
	GdkPixbuf *overlay;
	GdkPixbuf *pixbuf;
	gint width;
	gint height;
	gint x;
	gint y;

	gint segments;
	gint segment_size;
	gint horizontal;

	gint bands;
	gint band_size;

	gint slide;

	float *values;

	SpectrumPeakType peak_mode;
	float *peak_values;
	gfloat peak_fall;

	gint dynamic;
	gint dynamic_size;
	gint one_image;
};

#define SPECTRUM_BUFFER_SIZE 256

#define spectrum_set_peak_mode(x, y) (x->peak_mode = y)
#define spectrum_set_peak_fall(x, y) (x->peak_fall = CLAMP(y, 0.1, 10.0))


SpectrumData *spectrum_new(GdkPixbuf *pb, gint x, gint y, gint segments, gint bands,
			   gint horizontal, gint one_image, gint dynamic, gint slide);
SpectrumData *spectrum_new_from_data(gchar **data, gint x, gint y, gint segments, gint bands,
				     gint horizontal, gint one_image, gint dynamic, gint slide);
SpectrumData *spectrum_new_from_file(const gchar *file, gint x, gint y, gint segments, gint bands,
				     gint horizontal, gint one_image, gint dynamic, gint slide);
void spectrum_free(SpectrumData *spectrum);

WidgetData *spectrum_register(SkinData *skin, SpectrumData *spectrum, const gchar *key, const gchar *text_id);
WidgetData *spectrum_register_to_skin(const gchar *key, gchar **data, gint x, gint y,
				      gint segments, gint bands,
				      gint horizontal, gint one_image, gint dynamic, gint slide,
				      SkinData *skin, const gchar *text_id);

RegisterData *spectrum_register_key(const gchar *key, UIData *ui,
				 gint (*status_get_func)(SpectrumData *spectrum, const gchar *key, gpointer data), gpointer status_get_data,
				 void (*mode_get_func)(SpectrumData *spectrum, const gchar *key, gpointer data), gpointer mode_get_data);

gint spectrum_value_set(const gchar *key, UIData *ui, gint32 *buffer, gfloat fall_percent);
gint spectrum_value_all_one(const gchar *key, UIData *ui, gfloat value, gfloat fall_percent);

gint spectrum_reset(const gchar *key, UIData *ui);

gint spectrum_set_options(const gchar *key, UIData *ui, SpectrumPeakType mode, gfloat fall_speed);

/* use these to set the widget from the status callback,
 * return the value in the status function that is returned by these calls
 */
gint spectrum_widget_set(SpectrumData *spectrum, gfloat value);
gint spectrum_widget_reset(SpectrumData *spectrum);
gint spectrum_widget_set_buffer(SpectrumData *spectrum, gint32 *buffer);


void spectrum_sync_band_count(SpectrumData *spectrum, gint bands);

WidgetType spectrum_type_id(void);
void spectrum_type_init(void);


#endif

